/*
 * q_edf.c	EDF queue control.
 *
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Authors:	Dimitry Ketov <dimitry.ketov@avalon.ru>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>

#include "utils.h"
#include "tc_util.h"

struct tc_edf_qopt
{
    unsigned delay;
    unsigned limit;
};

struct tc_edf_copt
{
    unsigned delay;
    unsigned limit;
};

static void explain_class(void)
{
        fprintf(stderr, "Usage: ... edf delay DELAY [ limit NUMBER ]\n");
}

static void explain(void)
{
	fprintf(stderr, "Usage: ... edf [ delay DELAY ] [ limit NUMBER ]\n");
}

#define usage() return(-1)

static int edf_parse_class_opt(struct qdisc_util *qu, int argc, char **argv, struct nlmsghdr *n)
{
	int ok = 0;
	struct tc_edf_copt opt;
	memset(&opt, 0, sizeof(opt));

	while (argc > 0) {
		if (strcmp(*argv, "delay") == 0) {
			NEXT_ARG();
			if (get_usecs(&opt.delay, *argv)) {
				fprintf(stderr, "Illegal \"delay\"\n");
				return -1;
			}
			ok++;
		} else if (strcmp(*argv, "limit") == 0) {
			NEXT_ARG();
			if (get_size(&opt.limit, *argv)) {
				fprintf(stderr, "Illegal \"limit\"\n");
				return -1;
			}
		} else if (strcmp(*argv, "help") == 0) {
			explain_class();
			return -1;
		} else {
			fprintf(stderr, "What is \"%s\"?\n", *argv);
			explain_class();
			return -1;
		}
		argc--; argv++;
	}

 	if (ok)
	{
		addattr_l(n, 1024, TCA_OPTIONS, &opt, sizeof(opt));
		return 0;
	}
	else
	{
		explain_class();
		return -1;
	}
}

static int edf_parse_opt(struct qdisc_util *qu, int argc, char **argv, struct nlmsghdr *n)
{
	struct tc_edf_qopt opt;
	memset(&opt, 0, sizeof(opt));

	while (argc > 0) {
		if (strcmp(*argv, "delay") == 0) {
			NEXT_ARG();
			if (get_usecs(&opt.delay, *argv)) {
				fprintf(stderr, "Illegal \"delay\"\n");
				return -1;
			}
		} else if (strcmp(*argv, "limit") == 0) {
			NEXT_ARG();
			if (get_size(&opt.limit, *argv)) {
				fprintf(stderr, "Illegal \"limit\"\n");
				return -1;
			}
		} else if (strcmp(*argv, "help") == 0) {
			explain();
			return -1;
		} else {
			fprintf(stderr, "What is \"%s\"?\n", *argv);
			explain();
			return -1;
		}
		argc--; argv++;
	}

	addattr_l(n, 1024, TCA_OPTIONS, &opt, sizeof(opt));
	return 0;
}

static int edf_print_opt(struct qdisc_util *qu, FILE *f, struct rtattr *opt)
{
	struct tc_edf_qopt *qopt;

	if (opt == NULL)
		return 0;

	if (RTA_PAYLOAD(opt)  < sizeof(*qopt))
		return -1;
	qopt = RTA_DATA(opt);

	fprintf(f, "limit %up ", qopt->limit);
	fprintf(f, "delay %uus", qopt->delay);
	return 0;
}

static int edf_print_class_opt(struct qdisc_util *qu, FILE *f, struct rtattr *opt)
{
	struct tc_edf_copt *copt;

	if (opt == NULL)
		return 0;

	if (RTA_PAYLOAD(opt)  < sizeof(*copt))
		return -1;

	copt = RTA_DATA(opt);

	fprintf(f, "limit %up ", copt->limit);
	fprintf(f, "delay %uus", copt->delay);

	return 0;
}

static int edf_print_xstats(struct qdisc_util *qu, FILE *f, struct rtattr *xstats)
{
	printf("edf_print_xstats\n");

	return 0;
}

struct qdisc_util edf_util = {
	NULL,
	"edf",
	edf_parse_opt,
	edf_print_opt,
	edf_print_xstats,
	edf_parse_class_opt,
	edf_print_class_opt
};
